/ If you know ASM and understand this. Feel free to tinker and add functions. If you don't
/ know ASM , I would recommend just skipping this file and remember to include it right
/ after your Title statement.

/ A word to ASM buffs: When you look at the resulting source code, you will see several places for
/ optimization (mostly around assignments and several math statements next to each other). Feel
/ free to optimize but don't do it until you have your final version. The compiler will
/ just write over your precious optimizations.

/ Also send any additional useful System Functions you write or add (you can tell I am lacking in
/ lots of areas, mainly many gfx routines) In fact, email me when you have incorporated sprites
/ with clipping, , any link routines THAT WORK, sound stuff as well as advanced math stuff you
/ may come up with (simple trig functions, 16-bit multiplication, etc.)

SystemFunction(ClearScreen)
CLEARSCREEN:
;clears the home screen
    ld hl, VIDEOBUFF
    ld de, VIDEOBUFF+1
    ld (hl),0
    ld bc, NUMSCREENBYTES-1
    ldir
    jp ionFastCopy
EndSystemFunction

SystemFunction(CopyBuffer)
COPYBUFFER .equ ionFastCopy
EndSystemFunction

SystemFunction(WaitForSecond)
WAITFORSECOND:
;waits for 2nd to be pressed and released
    call TI_wfsb4
TI_wfsb2:
    call TI_wfsb3
    jr nz,TI_wfsb2
TI_wfsb4:
    call TI_wfsb3
    jr z,TI_wfsb4
    ret
TI_wfsb3:
    ld a, $ff
    out (1),a
    ld a, $bf
    out (1),a
    in a,(1)
    cp 223
    ret
EndSystemFunction

SystemFunction(DrawString, syspointerarg0, sysbytearg0, sysbytearg1)
DRAWSTRING:
;draws a string
    ld hl,(SYSBYTEARG0)
    ld (pencol), hl
    ld hl, (SYSPOINTERARG0)
    bcall _vputs
    ret
EndSystemFunction

SystemFunction(DrawNumber, syswordarg0, sysbytearg0, sysbytearg1)
DRAWNUMBER:
;draws a 16-bit number
    ld hl,(SYSBYTEARG0)
    ld (pencol),hl
    ld hl,(SYSWORDARG0)
    ld de,op1+5
    xor a
    ld (de),a
TI_DNLOOP:
    bcall _divhlby10
    dec de
    add  a,'0'
    ld (de),a
    ld a,h
    or l
    jr nz,TI_DNLOOP
    ex de,hl
    bcall _vputs
    ret
EndSystemFunction

SystemFunction(DrawBigText, syspointerarg0, sysbytearg0, sysbytearg1)
DRAWBIGTEXT:
; draws in big text
    ld hl,(SYSBYTEARG0)
    ld (currow), hl
    ld hl,(SYSPOINTERARG0)
    bcall _puts
    ret
EndSystemFunction

SystemFunction(InvertMem, syswordarg0, syspointerarg0)
INVERTMEM:
; inverts a large amount of memory (for gfx purposes)
    ld hl,(SYSPOINTERARG0)
    ld de,(SYSWORDARG0)
TI_INVERTMEMLOOP:
    ld a,(hl)
    cpl
    ld (hl),a
    inc hl
    dec de
    ld a, d
    or e
    jr nz, TI_INVERTMEMLOOP
    ret
EndSystemFunction

SystemFunction(LoadString, syspointerarg0, syspointerarg1)
LOADSTRING:
;loads a string from memory
    ld hl,(SYSPOINTERARG0)
    ld de, op1
    ld bc, 10
    ldir
    bcall _chkfindsym
    jr c, TI_LSERROR
    ex de,hl          ; exchange de and hl
    ld c,(hl)         ; get LSB of the size
    inc hl            ; next byte
    ld b,(hl)         ; get MSB of the size
    inc hl            ; next byte
    inc hl	; since it is actually a program. We skip the $bb and 6d
    inc hl
    ld de,(SYSPOINTERARG1)       ; buffer
    ldir              ; copy data
    xor a
TI_LOADSTRING_FINISH:
    ld (SUCCESS),a
    ret
TI_LSERROR:
    ld a,1
    jr TI_LOADSTRING_FINISH
EndSystemFunction

/ This was a VERY interesting idea!
SystemFunction(LoadLibrary, syspointerarg0)
LOADLIBRARY:
;loads a library into the last 4k of the RAM_PAGE_1 (assuming 14k is alterable)
    SysFuncRequire(LoadString)
    ld hl, $b800
    ld (SYSPOINTERARG1), hl
    call LOADSTRING
    ld a, (SUCCESS) ;if library succeeded in loading
    or a
    ret z
;disables any user interrupts being used
    res 2,(iy+$23)
    ld a,$3c
    out (0),a    
;draws error string
    ld hl, 0
    ld (curRow), hl
    ld hl, TI_LOADLERRSTRING
    bcall _puts
    ret
TI_LOADLERRSTRING:
    .db "Required Library was not found.", 0
EndSystemFunction

SystemFunction(GetRandomNumber, sysbytearg0)
GETRANDOMNUMBER:
	ld a,r
	ld e,a
	ld d,0
	ld hl, (GETRANDOMNUMBER_SEED)
	add hl,de
	ld (GETRANDOMNUMBER_SEED),hl
	ld a,(SYSBYTEARG0)
	bcall _divHLbyA
	inc a
	ld (SUCCESS),a
	ret
GETRANDOMNUMBER_SEED:
	.dw $3B56	; just some random number I chose to start out with.
EndSystemFunction

SystemFunction(ScrambleRandomSeed)
	SysFuncRequire(GetRandomNumber)
SCRAMBLERANDOMSEED:
	ld a,r
	rrca
	inc a
	ld (GETRANDOMNUMBER_SEED+1),a
	ret
EndSystemFunction