/ If you know ASM and understand this. Feel free to tinker and add functions. If you don't
/ know ASM , I would recommend just skipping this file and remember to include it right
/ after your Title statement.

/ A word to ASM buffs: When you look at the resulting source code, you will see several places for
/ optimization (mostly around assignments and several math statements next to each other). Feel
/ free to optimize but don't do it until you have your final version. The compiler will
/ just write over your precious optimizations.

/ Also send any additional useful System Functions you write or add (you can tell I am lacking in
/ lots of areas, mainly many gfx routines) In fact, email me when you have incorporated sprites
/ with clipping, grayscale sprite routines, any link routines THAT WORK, sound stuff as well as
/ advanced math stuff you may come up with (simple trig functions, 16-bit multiplication, etc.)

SystemFunction(ClearScreen)
CLEARSCREEN:
;clears the home screen
    ld hl,$fc00
    ld de, $fc01
    ld (hl),l
    ld bc,1023
    ldir
    ret
EndSystemFunction

SystemFunction(WaitForSecond)
WAITFORSECOND:
    ld bc,$3f20
    call TI_wfsb4
;waits for 2nd to be pressed and released
TI_wfsb2:
    call TI_GETKEY
    or a
    jr nz,TI_wfsb2
TI_wfsb4:
    call TI_GETKEY
    or a
    jr z,TI_wfsb4
    ret
EndSystemFunction

SystemFunction(DrawString, syspointerarg0, sysbytearg0, sysbytearg1)
DRAWSTRING:
;draws a string
    ld hl,(SYSBYTEARG0)
    ld (_penCol), hl
    ld hl, (SYSPOINTERARG0)
    jp _vputs
EndSystemFunction

SystemFunction(DrawNumber, syswordarg0, sysbytearg0, sysbytearg1)
DRAWNUMBER:
;draws a 16-bit number
    ld hl,(SYSBYTEARG0)
    ld (_penCol),hl
    ld hl,(SYSWORDARG0)
    ld de,_OP1+5
    xor a
    ld (de),a
TI_DNLOOP:
    call _divHLby10
    dec de
    add  a,'0'
    ld (de),a
    ld a,h
    or l
    jr nz,TI_DNLOOP
    ex de,hl
    jp _vputs
EndSystemFunction

SystemFunction(SaveString, syspointerarg0, syspointerarg1, syswordarg0)
SAVESTRING:
; saves memory into a string
; first we delete the string
    ld hl, (SYSPOINTERARG0)
    push hl
    rst 20h
    rst 10h
    jr c, TI_SSNOEXIST
    call _delvar
TI_SSNOEXIST:
; then we create the string
    call _ram_page_1
    pop hl
    rst 20h
    ld hl, (SYSWORDARG0)
    push hl
    call _createstrng
; and then we copy the data
    call _ram_page_1
    ld hl,(SYSPOINTERARG1)
    push hl
    ld hl,(SYSPOINTERARG0)
    rst 20h
    rst 10h
    ld a,b
    ex de,hl
    call _ahl_plus_2_pg3
    call _set_abs_dest_addr
    pop hl
    ld a,h
    sub $80
    ld h,a
    ld a,1
    call _set_abs_src_addr
    xor a
    ld (SUCCESS),a
    pop hl
    call _set_mm_num_bytes
    call _mm_ldir
    jp _ram_page_1
EndSystemFunction

SystemFunction(LoadString, syspointerarg0, syspointerarg1, syswordarg0)
LOADSTRING:
;loads a string from memory
    ld hl,(SYSWORDARG0)
    push hl
    ld hl,(SYSPOINTERARG1)
    push hl
    ld hl,(SYSPOINTERARG0)
    rst 20h
    rst 10h
    jr c, TI_LSERROR
    ld a,b
    ex de,hl
    call _ahl_plus_2_pg3
    call _set_abs_src_addr
    pop hl
    ld a,h
    sub $80
    ld h,a
    ld a,1
    call _set_abs_dest_addr
    pop hl
    xor a
    call _set_mm_num_bytes
    call _mm_ldir
    xor a
TI_LOADSTRING_FINISH:
    push af
    call _ram_page_1
    pop af
    ld (SUCCESS),a
    ret
TI_LSERROR:
    pop hl
    pop hl
    ld a,1
    jr TI_LOADSTRING_FINISH
EndSystemFunction

/ This was a VERY interesting idea!
SystemFunction(LoadLibrary, syspointerarg0, syswordarg0)
LOADLIBRARY:
;loads a library into the last 4k of the RAM_PAGE_1 (assuming 14k is alterable)
    SysFuncRequire(LoadString)
    ld hl, $A900
    ld (SYSPOINTERARG1),hl
    call LOADSTRING
    ld a, (SUCCESS) ;if library succeeded in loading
    or a
    ret z
;disables any user interrupts being used
    res 2,(iy+$23)
    ld a,$3c
    out (0),a    
;draws error string
    ld hl, 0
    ld (_curRow), hl
    ld hl, TI_LOADLERRSTRING
    call _puts
    jp _jforcecmdnochar
TI_LOADLERRSTRING:
    .db "Required Library was not found.", 0
EndSystemFunction

SystemFunction(EnableInterrupt, syspointerarg0)
ENABLEINTERRUPT:
;enables a user interrupt routine
    di
    ld hl,(SYSPOINTERARG0)
    ld de,_alt_interrupt_exec
    ld bc,200
    ldir
    ld de,$28
    ld a,(_alt_interrupt_exec)
    ld hl,_alt_int_chksum + $28
    add a,(hl)
    add hl,de
    add a,(hl)
    add hl,de
    add a,(hl)
    add hl,de
    add a,(hl)
    add hl,de
    add a,(hl)
    ld (_alt_int_chksum),a
    set 2,(iy+$23)
    ei
    ret
EndSystemFunction

SystemFunction(DisableInterrupt)
DISABLEINTERRUPT:
;disables user interrupts
    res 2,(iy+$23)
    ld a,$3c
    out (0),a
    set graphdraw,(iy+graphflags)
    ret
EndSystemFunction

SystemFunction(DrawBigText, syspointerarg0, sysbytearg0, sysbytearg1)
DRAWBIGTEXT:
; draws in big text
    ld hl,(SYSBYTEARG0)
    ld (_curRow), hl
    ld hl,(SYSPOINTERARG0)
    jp _puts
EndSystemFunction

SystemFunction(InvertMem, syswordarg0, syspointerarg0)
INVERTMEM:
; inverts a large amount of memory (for gfx purposes)
    ld hl,(SYSPOINTERARG0)
    ld de,(SYSWORDARG0)
TI_INVERTMEMLOOP:
    ld a,(hl)
    cpl
    ld (hl),a
    inc hl
    dec de
    ld a, d
    or e
    jr nz, TI_INVERTMEMLOOP
    ret
EndSystemFunction

SystemFunction(InputString, sysbytearg0, sysbytearg1, sysbytearg2, syspointerarg0)
INPUTSTRING:
; sysbytearg2 is max stringlength
; syspointerarg0 is stringbuffer
    ld hl,(SYSPOINTERARG0)
    ld (TI_tempvar), hl
    ld a,(SYSBYTEARG2)
    ld b, a
TI_cnamelp2
    ld (hl), ' '
    inc hl
    djnz TI_cnamelp2
TI_cnameloop:
    ld hl, (SYSBYTEARG0)
    ld (_penCol), hl
    ld hl, (SYSPOINTERARG0)
    call _vputs
TI_getkey
    call _getky
    or a
    jr z, TI_getkey
    cp K_DEL
    jr z, TI_deletehighscore
    cp K_EXIT
    ret z
    cp K_ENTER
    ret z
    dec a
    ld e, a
    ld d, 0
    ld hl, TI_stringinputmatrix
    add hl, de

    ld a, (hl)
    ld hl, (TI_tempvar)
    ld (hl), a
    inc hl
    ld a, (hl)
    or a
    jr z, TI_cnameloop
    ld (TI_tempvar), hl
    jr TI_cnameloop
TI_deletehighscore
    ld a, (_penCol)
    sub 6
    ld (_penCol), a
    ld hl, TI_blankstring
    call _vputs
    ld hl, (TI_tempvar)
    ld a, (SYSPOINTERARG0)
    cp l
    jr z, TI_getkey
    ld (hl), ' '
    dec hl
    ld (TI_tempvar), hl
    jr TI_cnameloop
TI_tempvar:
    .dw 0
TI_stringinputmatrix
    ;11-20:
    .db "*{}^    ?XTOJE0  WSNID9 ZVRMHC8 "
    .db "YUQLGB7!?=PKFA6-54321  ."
TI_blankstring
    .db "     ",0
EndSystemFunction

SystemFunction(ContrastUp)
CONTRASTUP:
    ld hl, $c008
    inc (hl)
    ld a,(hl)
    out (2),a
    ret
EndSystemFunction

SystemFunction(ContrastDown)
CONTRASTDOWN:
    ld hl, $c008
    dec (hl)
    ld a,(hl)
    out (2),a
    ret
EndSystemFunction

SystemFunction(GetContrast)
GETCONTRAST:
    ld a,($c008)
    ld (SUCCESS), a
    ret
EndSystemFunction

SystemFunction(SetContrast, sysbytearg0)
SETCONTRAST:
	ld a, (SYSBYTEARG0)
	ld ($c008), a
 	out (2),a
	ret
EndSystemFunction

/ Retrieves X variable's value and stores is in syswordarg0.
/ This and GetRealY are for those unfortunate TI-BASIC people
SystemFunction(GetRealX)
GETREALX:
	call _rclX
GETREAL_END:
	call _ram_page_1
; first we load $c08a into a (this holds exponent of variable)
	ld hl, 0
	ld a, ($c08a)
	cp 6
	jr nc, GETREAL_ERROR
	ld ix, $c08c
	ld b,a
	inc b
	ld d,$f0
GETREAL_LOOP:
	push bc
	call _mulHL10
	ld e,(ix+0)
	ld a,d
	ld d,$0f
	cp $f0
	jr z, GETREAL_FIRSTBCD
	ld a,$0f
	ld d,$f0
	inc ix
GETREAL_FIRSTBCD:
	and e
	cp 16
	jr c, GETREAL_NOROTATE
	rrca
	rrca
	rrca
	rrca
GETREAL_NOROTATE:
	ld c,a
	ld b,0
	add hl,bc
	pop bc
	djnz GETREAL_LOOP
GETREAL_ERROR:
	ld (SYSWORDARG0), hl
	ld a, l
	ld (SYSBYTEARG0), a
	ret
EndSystemFunction

SystemFunction(GetRealY)
	SysFuncRequire(GetRealX)
GETREALY:
	call _rcly
	jr GETREAL_END
EndSystemFunction

SystemFunction(GetRandomNumber, sysbytearg0)
GETRANDOMNUMBER:
	ld a,r
	ld e,a
	ld d,a
	ld hl, (GETRANDOMNUMBER_SEED)
	add hl,de
	ld (GETRANDOMNUMBER_SEED),hl
	ld a,(SYSBYTEARG0)
	call _divHLbyA
	inc a
	ld (SUCCESS),a
	ret
GETRANDOMNUMBER_SEED:
	.dw $3B56	; just some random number I chose to start out with.
EndSystemFunction

SystemFunction(ScrambleRandomSeed)
	SysFuncRequire(GetRandomNumber)
SCRAMBLERANDOMSEED:
	ld a,r
	rrca
	inc a
	ld (GETRANDOMNUMBER_SEED+1),a
	ret
EndSystemFunction