    .plugin string86
#include "ti86asm.inc"
SYSPOINTERARG0                           .equ $8100
SYSPOINTERARG1                           .equ $8102
SYSBYTEARG0                              .equ $8104
SYSBYTEARG1                              .equ $8105
SYSBYTEARG2                              .equ $8106
SYSBYTEARG3                              .equ $8107
SYSBYTEARG4                              .equ $8108
SUCCESS                                  .equ $8109
SPRITE8HEIGHT                            .equ $810a
SPRITE16HEIGHT                           .equ $810b
SYSWORDARG0                              .equ $810c
SYSWORDARG1                              .equ $810e
VIDEOBUFF                                .equ $8110
.org $A900

; This is the FIRST external library for TIPower!
;  NOTE: This library has little interest in speed (it's still kinda fast, but doesn't waste space
;                                                   to make it lightning)

; There are 8 functions included within the library..
;   DrawWindowRounded(sysbytearg0 = left, sysbytearg1=top, sysbytearg2=width, sysbytearg3=height)
;       Draws a nice-looking window with rounded corners. Width and Height both must be
;       at least 5. Anything less will yield graphical errors
;   DrawWindowPlain
;       Same as above except with square corners
;   DrawWindowTitleRounded(same as above..., syspointerarg0 = titlestring)
;       This performs the same as above except it draws a nice title bar. height must be 10 though.
;   DrawWindowTitlePlain ... etc.etc.
;   MessageBox(left, top, width, height, syspointerarg0 = title, syspointerarg1=messagestrings, sysbytearg4 = type)
;   MessageBoxCenter(width, height, title, messagestrings, type)
;           VALID TYPES:
;               MB_NONE (Doesn't wait for user input)
;               MB_OK   ('2nd' button used to quit)
;               MB_YESNO (Press '2nd' then SUCCESS=IDYES, Alpha = IDNO)
;               MB_YESNOCANCEL (2nd=IDYES, alpha=IDNO, exit=IDCANCEL)
;           Determining Message Box Height : height = 10 + numlines * 6
;   HelpBox(title, messagestrings)
;           One of handiest of the functions, this will display multiple pages of info
;           in a window, and the user can use 2nd and alpha to scroll up and down, and exit to leave.
;   WindowMenu(width, height, title, syspointerarg1 = menudata)
;           This is THE handiest function in the library!
;           Menu data is in the following format:
;		[	.db 255
;			.dw PREINIT ]  optional preinitialization function (what to do BEFORE window is drawn
;               .db number of menu options
;               .db null-terminating strings (for each menu option's text)
;		[	.db 255
;			.dw POSTINIT ] postinitialization function so you can add things to the window
;               .dw functionnames (corresponding to the strings) in ALL CAPS

; *************************************************************************************************
DRAWWINDOWROUNDED: ; 123 bytes!
    call LIBTI_FINDPIXEL       ;hl now equals video memory location
                            ;(accounting for buffer as well) and a is the bit
;first we draw our top line:
    call DRAWWINDOW_HORZLINE ;draw the top line
    call DRAWWINDOW_FILLTYPE1 ;and make rounded corner..
    ld bc, (SYSBYTEARG2)    ;we need the height - 4 to account for top and bottom lines
    dec b
    dec b                   ;we need to keep value in 'a'. This is the fastest
    dec b                   ;method possible without losing it.
    dec b    
DRAWWINDOWROUNDED_MAINLOOP:
    push bc
    call DRAWWINDOW_FILLTYPE2   ;fill in height-4 lines
    pop bc
    djnz DRAWWINDOWROUNDED_MAINLOOP
    
    call DRAWWINDOW_FILLTYPE1 ;draw bottom rounded corner
    jp DRAWWINDOW_HORZLINE    ; and bottom line (jump because we are done!)
    
DRAWWINDOWPLAIN:
    CALL LIBTI_FINDPIXEL
    CALL DRAWWINDOW_FULLLINE
    ld bc, (SYSBYTEARG2)
    dec b
    dec b
DRAWWINDOWPLAIN_MAINLOOP:
    push bc
    call DRAWWINDOW_FILLTYPE2
    pop bc
    djnz DRAWWINDOWPLAIN_MAINLOOP
    jp DRAWWINDOW_FULLLINE

; Draw Window Subroutines
DRAWWINDOW_HORZLINE:
    push hl
    ld e, a
    call DWSKIPAPIXEL ;this skips one pixel ahead. source at bottom of this section
    call DWSKIPAPIXEL ;we skip pixels so underneath the window looks neater.
    ld bc, (SYSBYTEARG1)
    dec b
    dec b
    dec b
    dec b
DRAWWINDOW_HORZLINE_LOOP:
    CALL DWDRAWAPIXEL
    djnz DRAWWINDOW_HORZLINE_LOOP
DRAWWINDOW_COM_ENDING:    ;all the DRAWWINDOW line routines end with this ending
    ld a,e
    ld de, 16
    pop hl
    add hl,de             ;go to the next row
    ret
DRAWWINDOW_FILLTYPE1:
    push hl
    ld e, a
    call DWSKIPAPIXEL
    call DWDRAWAPIXEL
    ld bc, (SYSBYTEARG1)
    dec b
    dec b
    dec b
    dec b
DRAWWINDOW_FILLTYPE1_LOOP:
    call DWERASEAPIXEL
    djnz DRAWWINDOW_FILLTYPE1_LOOP
    call DWDRAWAPIXEL
    jp DRAWWINDOW_COM_ENDING
DRAWWINDOW_FILLTYPE2:
    push hl
    ld e, a
    call DWDRAWAPIXEL
    ld bc, (SYSBYTEARG1)
    dec b
    dec b
DRAWWINDOW_FILLTYPE2_LOOP:
    call DWERASEAPIXEL
    djnz DRAWWINDOW_FILLTYPE2_LOOP
    call DWDRAWAPIXEL
    jp DRAWWINDOW_COM_ENDING
DRAWWINDOW_FILLTYPE3:
    push hl
    ld e, a
    call DWSKIPAPIXEL
    ld bc, (SYSBYTEARG1)
    dec b
    dec b
DRAWWINDOW_FILLTYPE3_LOOP:
    call DWDRAWAPIXEL
    djnz DRAWWINDOW_FILLTYPE3_LOOP
    jp DRAWWINDOW_COM_ENDING    
DRAWWINDOW_FULLLINE:
    push hl
    ld e,a
    ld bc, (SYSBYTEARG1)
DRAWWINDOW_FULLLINE_LOOP:
    call DWDRAWAPIXEL
    djnz DRAWWINDOW_FULLLINE_LOOP
    jp DRAWWINDOW_COM_ENDING
; there's something here everyone should pay attention to. notice I have DWDRAWAPIXEL jump
; back to finish at DWSKIPAPIXEL, and DWERASEAPIXEL gets the speed advantage by running straight
; into it. It gets this because in window drawing there is a LOT more erasing going on than pixel
; drawing. You gotta keep this stuff in mind when you are optimizing!
DWERASEAPIXEL:
    ld d,a
    cpl
    and (hl)
    ld (hl),a
    ld a,d
DWSKIPAPIXEL:
    rrca
    ret nc
    inc hl
    ret
DWDRAWAPIXEL:
    ld d,a
    or (hl)
    ld (hl),a
    ld a,d
    jp DWSKIPAPIXEL
;**************************************************************************************************
DRAWWINDOWTITLEROUNDED:

    call LIBTI_FINDPIXEL       ;hl now equals video memory location
                            ;(accounting for buffer as well) and a is the bit
;first we draw our top line:
    call DRAWWINDOW_HORZLINE ;draw the top line
    call DRAWWINDOW_FILLTYPE3 ;and make rounded corner..
    ld b,5
DRAWWINDOWTITLEROUNDED_MAINLOOPA:
    push bc
    call DRAWWINDOW_FULLLINE
    pop bc
    djnz DRAWWINDOWTITLEROUNDED_MAINLOOPA
    push af
    ld a, (SYSBYTEARG3)    ;we need the height - 4 to account for top and bottom lines
    sub 9
    ld b,a    
    pop af
DRAWWINDOWTITLEROUNDED_MAINLOOPB:
    push bc
    call DRAWWINDOW_FILLTYPE2   ;fill in height-4 lines
    pop bc
    djnz DRAWWINDOWTITLEROUNDED_MAINLOOPB
    
    call DRAWWINDOW_FILLTYPE1 ;draw bottom rounded corner
    call DRAWWINDOW_HORZLINE    ; and bottom line (jump because we are done!)    call DRAWWINDOWROUNDED
FINISH_WINDOWTITLE:
    ld de,(SYSBYTEARG0)
    inc e
    inc e
    inc e
    ld hl,(SYSPOINTERARG0)
    ld (_penCol), de
    ld a,(iy+textflags) ;saves whether the text inversion is on right now
    set textInverse,(iy+textflags) ; turns on text inversion
    call _vputs
    ld (iy+textflags),a ;reload our old settings
    ret
    
DRAWWINDOWTITLEPLAIN:
    call LIBTI_FINDPIXEL       ;hl now equals video memory location
                            ;(accounting for buffer as well) and a is the bit
;first we draw our top line:
    call DRAWWINDOW_FULLLINE ;draw the top line.
    ld b,6
DRAWWINDOWTITLEPLAIN_MAINLOOPA:
    push bc
    call DRAWWINDOW_FULLLINE
    pop bc
    djnz DRAWWINDOWTITLEPLAIN_MAINLOOPA
    push af
    ld a, (SYSBYTEARG3)    ;we need the height
    sub 8
    ld b,a    
    pop af
DRAWWINDOWTITLEPLAIN_MAINLOOPB:
    push bc
    call DRAWWINDOW_FILLTYPE2   ;fill in height-4 lines
    pop bc
    djnz DRAWWINDOWTITLEPLAIN_MAINLOOPB
    
    call DRAWWINDOW_FULLLINE ; and bottom line (jump because we are done!)
    jr FINISH_WINDOWTITLE

;**************************************************************************************************
#define LIBIDYES 1
#define LIBIDNO 2
#define LIBIDCANCEL 3
#define LIBIDUP 4
#define LIBIDDOWN 5
#define LIBMB_OK 0
#define LIBMB_YESNO 1
#define LIBMB_YESNOCANCEL 2
#define LIBMB_NONE 3
MESSAGEBOX:  ;92 bytes (for all message-box functions)
    CALL DRAWWINDOWTITLEROUNDED
    ld de,(SYSBYTEARG0)
    ld hl,515
    add hl,de
    ex de,hl
    ld hl,(SYSPOINTERARG1)
MESSAGEBOX_MAINLOOP:
    ld a,d
    add a,6
    ld d,a
    ld (_penCol), de
    call _vputs
    ld a,(hl)
    or a
    jr nz,MESSAGEBOX_MAINLOOP
    ld a,(SYSBYTEARG4)
    cp LIBMB_NONE
    ret z
    cp LIBMB_OK
    jr z, MESSAGEBOX_MB_OK
    cp LIBMB_YESNO
    jr z, MESSAGEBOX_MB_YESNO
MESSAGE_MB_YESNOCANCELLOOP:
    call TIPwin86KEYS
    ld a,b
    or a
    jr z, MESSAGE_MB_YESNOCANCELLOOP
MESSAGEBOX_COM_END:
    ld (SUCCESS), a
MESSAGEBOX_COM_END_LOOP:
    call TIPwin86KEYS
    ld a,b
    or a
    jr nz, MESSAGEBOX_COM_END_LOOP
    ret
MESSAGEBOX_MB_OK:
    call TIPwin86KEYS
    ld a,b
    cp LIBIDYES
    jr nz,MESSAGEBOX_MB_OK
    jr MESSAGEBOX_COM_END
MESSAGEBOX_MB_YESNO
    call TIPwin86KEYS
    ld a,b
    cp LIBIDCANCEL
    jr z,MESSAGEBOX_MB_YESNO
    or a
    jr z,MESSAGEBOX_MB_YESNO
    jr MESSAGEBOX_COM_END
MESSAGEBOXCENTER:
    ld a,255
    ld (SYSBYTEARG0), a
    jr MESSAGEBOX
    
;**************************************************************************************************
HELPWINDOW: ;pretty nice for 179 bytes!
    call MESSAGEBOX_COM_END_LOOP
    ld hl,(SYSPOINTERARG1)
    ld (HELPWINDOW_FIRSTLINE),hl
    ld hl,$0000
    ld (SYSBYTEARG0), hl
    ld hl,$4080
    ld (SYSBYTEARG2), hl
    CALL DRAWWINDOWTITLEROUNDED
HELPWINDOW_DRAW:
    ld de,$0203
    ld b,8
    ld hl,(SYSPOINTERARG1)
HELPWINDOWBOX_MAINLOOP:
    call HELPWINDOW_INITNEXTLINE
    call _vputs
    ld a,b
    cp 8
    jr nz, HELPWINDOW_NOSETNEXT
    ld (HELPWINDOW_NEXTLINE),hl
HELPWINDOW_NOSETNEXT:
    ld (HELPWINDOW_LASTLINE),hl
    ld a,(hl)
    or a
    jr z,HELPWINDOW_MAINLOOP_EXIT
    djnz HELPWINDOWBOX_MAINLOOP
HELPWINDOW_MAINLOOP_EXIT:
    call HELPWINDOW_INITNEXTLINE
HELPWINDOW_MAINLOOP_EXITLOOP:
    call TIPwin86KEYS
    ld hl, HELPWINDOW_DRAW
    push hl
    ld a,b
    cp LIBIDUP
    jr z, HELPWINDOW_GOUP
    cp LIBIDDOWN
    jr z, HELPWINDOW_GODOWN
    pop hl
    cp LIBIDCANCEL
    ret z
    jr HELPWINDOW_MAINLOOP_EXITLOOP
HELPWINDOW_GOUP:
    ld hl,(SYSPOINTERARG1)
    ld de,(HELPWINDOW_FIRSTLINE)
    call LIBTI_CP16
    ret z
    dec hl
    dec de
HELPWINDOW_GOUPLOOP:
    ld (SYSPOINTERARG1),hl
    dec hl
    ld a,(hl)
    or a
    ret z
    CALL LIBTI_CP16
    ret z
    jr HELPWINDOW_GOUPLOOP
HELPWINDOW_GODOWN:
    ld hl,(HELPWINDOW_NEXTLINE)
    ld de,(HELPWINDOW_LASTLINE)
    call LIBTI_CP16
    ret z    
    ld (SYSPOINTERARG1), hl
    ret
HELPWINDOW_INITNEXTLINE:
    ld a,d
    add a,6
    ld d,a
    ld (_penCol), de
HELPWINDOW_ERASELINE:
    push hl
    push de
    push bc
    ld l,d
    ld h,0
    add hl,hl
    add hl,hl
    add hl,hl
    add hl,hl
    ld de,(VIDEOBUFF)
    add hl,de
    ex de,hl
    ld a,6
HELPWINDOW_ERASELINE_LOOP:
    ld hl,HELPWINDOW_ERASELINE_DATA
    ld bc,16
    ldir
    dec a
    jr nz, HELPWINDOW_ERASELINE_LOOP
    pop bc
    pop de
    pop hl
    ret
HELPWINDOW_ERASELINE_DATA:
    .db 128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
HELPWINDOW_FIRSTLINE:
    .dw 0
HELPWINDOW_NEXTLINE:
    .dw 0
HELPWINDOW_LASTLINE:
    .dw 0
;**************************************************************************************************
WINDOWMENU: ;worth every 201 bytes!
    ld hl, SYSBYTEARG0
    ld a,255
    ld (hl), a

    ld hl, SYSPOINTERARG0
    ld de, WINDOWMENU_ARGUMENTDATA
    call WINDOWMENU_LDIR

WINDOWMENU_MAIN:
    call MESSAGEBOX_COM_END_LOOP	; makes sure a key isn't being pressed

    ld de,SYSPOINTERARG0
    ld hl,WINDOWMENU_ARGUMENTDATA
    call WINDOWMENU_LDIR
    
    ld hl,(SYSPOINTERARG1)
    ld a,(hl)
    cp 255
    call z, WINDOWMENU_INITIALIZATIONFUNCTION
    ld (SYSPOINTERARG1), hl

    call DRAWWINDOWTITLEROUNDED
    ld hl,(SYSPOINTERARG1)
    ld a,(hl)
    inc hl
    ld b,a
    dec a
    ld (WINDOWMENU_NUMOPTIONS), a
    push hl
    ld de, (SYSBYTEARG0)
    push de
    ld hl, 521
    add hl,de
    ex de,hl
    pop ix
    pop hl
    push ix
WINDOWMENU_DRAWLOOP:
    ld a,d
    add a,6
    ld d,a
    ld (_penCol), de
    call _vputs
    djnz WINDOWMENU_DRAWLOOP
    ld a,(hl)
    cp 255
    call z, WINDOWMENU_INITIALIZATIONFUNCTION
    push hl
    pop ix
    pop de
    ld hl,$0804
    add hl,de
    ex de,hl
    xor a
    ld (WINDOWMENU_CURSELECT), a
WINDOWMENU_MAINLOOP:
    ld (_penCol),de
    ld hl,WINDOWMENU_CURSOR
    push de
    call _vputs
WINDOWMENU_KEYLOOP
    call TIPwin86KEYS
    ld a,b
    or a
    jr z,WINDOWMENU_KEYLOOP
    pop de
    push bc
    push de
    ld (_penCol),de
    call _vputs
WINDOWMENU_KEYLOOP2
    call TIPwin86KEYS
    ld a,b
    or a
    jr nz,WINDOWMENU_KEYLOOP2
    pop de
    pop af
    ld hl,WINDOWMENU_CURSELECT
    cp LIBIDUP
    jr z,WINDOWMENU_GOUP
    cp LIBIDDOWN
    jr z,WINDOWMENU_GODOWN
    cp LIBIDYES
    jr z,WINDOWMENU_SELECT
    cp LIBIDCANCEL
    ret z
    jr WINDOWMENU_MAINLOOP
WINDOWMENU_GOUP:
    ld a,(hl)
    or a
    jr z, WINDOWMENU_MAINLOOP
    dec (hl)
    ld a,d
    sub 6
    ld d,a
    jr WINDOWMENU_MAINLOOP
WINDOWMENU_GODOWN:
    ld a,(WINDOWMENU_NUMOPTIONS)
    cp (hl)
    jr z, WINDOWMENU_MAINLOOP
    inc (hl)
    ld a,d
    add a,6
    ld d,a
    jr WINDOWMENU_MAINLOOP
WINDOWMENU_SELECT:
    ld a,(hl)
    rlca
    push ix
    pop de
    ld l,a
    ld h,0
    add hl,de
    ld e,(hl)
    inc hl
    ld d,(hl)
    ld hl,WINDOWMENU_MAIN
    push hl
    push de
    ret
WINDOWMENU_LDIR:
    ld bc, 8
    ldir
    ret
WINDOWMENU_INITIALIZATIONFUNCTION:
	inc hl
	ld e, (hl)
	inc hl
	ld d, (hl)
	inc hl
	push hl
	ld (WINDOWMENU_INITIALIZATIONMORPHIC+1), de
WINDOWMENU_INITIALIZATIONMORPHIC
	call $0000
	pop hl
	ret
WINDOWMENU_CURSELECT:
    .db 0
WINDOWMENU_NUMOPTIONS:
    .db 0
WINDOWMENU_CURSOR:
    .db 5, 0
    .db "    ", 0
WINDOWMENU_ARGUMENTDATA:
    .db 0,0,0,0,0,0,0,0
;**************************************************************************************************
TIPwin86KEYS:
    ld d, 2
TIPwin86KEYS_LOOP:
    ld b, LIBIDYES
    ld a,63         ;check 2nd status
    out (1), a
    in a,(1)
    ld c,a
    and 32
    ret z
    ld b, LIBIDCANCEL
    ld a,c          ;check exit status
    and 64
    ret z
    ld b, LIBIDNO
    ld a,95         ;check alpha status
    out (1), a
    in a,(1)
    and 128
    ret z
    ld b, LIBIDUP
    ld a,$7e         ;check up status
    out (1), a
    in a,(1)
    and 8
    ret z
    ld b, LIBIDDOWN
    ld a,$7e         ;check down status
    out (1), a
    in a,(1)
    and 1
    ret z
    dec d
    jr nz, TIPwin86KEYS_LOOP
    ld b,0
    ret
        
LIBTI_CP16:
    push hl
    or a
    sbc hl,de
    pop hl
    ret    
    
; required TIPower System Function
LIBTI_FINDPIXEL:
; first we check if width = 255 (if it does, we center the box in the screen)
  ld de,(SYSBYTEARG0)
  ld a, e
  cp 255
  jr nz, START_TI_FINDPIXEL
    ; calculate and assign appropriate values
  ld hl,(SYSBYTEARG2)    ;get width, height
  sra h                 ;divide both by two
  sra l
  ld a,64       ;get correct left value ( 64 - width / 2)
  sub l
  ld l,a
  ld a,32       ;get correct top value ( 32 - height / 2)
  sub h
  ld h,a
  ld (SYSBYTEARG0), hl
  ex de,hl
START_TI_FINDPIXEL:
; typical find pixel routine
; Modified version of Dan Eble's and James Yopp's findpixel routine
  ld hl,LIBTI_FP_Bits
  ld a,e
  and $07         ; a = bit offset
  add a,l
  ld l,a
  adc a,h
  sub l
  ld h,a
  ld c,(hl)       ; c = bitmask for (hl)
  ld hl,LIBTI_FP_RLD
  ld (hl),d
  ld a,e          ; a = x/8 (byte offset within row)
  rrca
  rrca   
  rrca
  rld
  ld l,(hl)
  and %00000011
  ld h,a          ; hl -> byte in vid mem
  ld de, (VIDEOBUFF)
  add hl,de  
  ld a,c          ; now a = bitmask for (hl)
  ret
LIBTI_FP_RLD:  .db $00
LIBTI_FP_Bits: .db $80,$40,$20,$10,$08,$04,$02,$01